﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyClock
{
    public partial class DispReg : Form
    {
        public DispReg()
        {
            InitializeComponent();
            Initialize(); // 外観情報の初期化
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// 外観設定情報クラス
        /// </summary>
        public class Register
        {
            private static int m_reg_index;
            private static int m_reg_max_index;
            private static object[] a_reg = new object[12];
            private static ArrayList alReg = new ArrayList();

            private Color m_BackColor;
            private Font m_Font;
            private Color m_FontColor;
            private Size m_Size;
            private double m_Opacity;
            private BorderStyle m_Border;

            // <Ver 1.18.0>現在のインデックスにアクセスするプロパティ
            public int CurrentIndex
            {
                get { return m_reg_index; }
                set { m_reg_index = value; }
            }

            // <Ver 1.18.0>現在の最大インデックスにアクセスするプロパティ
            public int MaxIndex
            {
                get { return m_reg_max_index; }
                set { m_reg_max_index = value; }
            }

            // <Ver 1.18.0>現在のインデックス表示する文字列を返すクラスメンバ
            public string GetIndexString()
            { return "INDEX " + CurrentIndex.ToString() + "/" + MaxIndex.ToString(); }

            // <Ver 1.18.0>任意のインデックス表示する文字列を返すクラスメンバ
            public string SetIndexString(int index)
            {
                m_reg_index = index;
                return "INDEX " + CurrentIndex.ToString() + "/" + MaxIndex.ToString();
            }

            // <Ver 1.18.0>現在の外観プロパティ文字列を取得するクラスメンバ
            public void UpdatePropertyStrings()
            {
                m_BackColor = Display.DispInstance.DispBackColor;
                m_Font = Display.DispInstance.DispFont;
                m_FontColor = Display.DispInstance.DispForeColor;
                m_Size = Display.DispInstance.DispSize;
                m_Opacity = Display.DispInstance.DispOpacity;
                m_Border = Display.DispInstance.DispBorder;

                s_MaxIndex = MaxIndex.ToString();
                s_BackColor = m_BackColor.Name.ToString();
                s_Font_Name = m_Font.Name.ToString();
                s_Font_Size = m_Font.Size.ToString();
                s_Font_Style = m_Font.Style.ToString();
                s_Font_Unit = m_Font.Unit.ToString();
                s_Font_GdiCharSet = m_Font.GdiCharSet.ToString();
                s_Font_GdiVertialFont = m_Font.GdiVerticalFont.ToString();
                s_FontColor = m_FontColor.Name.ToString();
                s_Size_Width = m_Size.Width.ToString();
                s_Size_Height = m_Size.Height.ToString();
                s_Opacity = m_Opacity.ToString();
                s_Border = m_Border.ToString();
            }

            // <Ver 1.18.0>a_Reg[]をプロパティ文字列で更新するクラスメンバ
            private void SetaReg()
            {
                a_reg[0] = s_BackColor;
                a_reg[1] = s_Font_Name;
                a_reg[2] = s_Font_Size;
                a_reg[3] = s_Font_Style;
                a_reg[4] = s_Font_Unit;
                a_reg[5] = s_Font_GdiCharSet;
                a_reg[6] = s_Font_GdiVertialFont;
                a_reg[7] = s_FontColor;
                a_reg[8] = s_Size_Width;
                a_reg[9] = s_Size_Height;
                a_reg[10] = s_Opacity;
                a_reg[11] = s_Border;
            }

            // <Ver 1.18.0>indexで選択した個別の外観情報文字列をINIファイルから取得するクラスメンバ
            public void GetDispInfo(int index)
            {
                Config Conf = new Config();
                if (index > 0) { Conf.GetRegInfo(index); }
                else if (index == 0) { Conf.GetDefaultRegInfo(Display.m_normDisp); }
            }

            // <Ver 1.18.0>indexで選択した個別の外観プロパテｨを取得するクラスメンバ
            public void GetRegItem(int index)
            {
                GetDispInfo(index);

                m_BackColor = Color.FromName(s_BackColor);
                m_Font = new Font(
                    s_Font_Name,
                    float.Parse(s_Font_Size),
                    (FontStyle)Enum.Parse(typeof(FontStyle), s_Font_Style),
                    (GraphicsUnit)Enum.Parse(typeof(GraphicsUnit), s_Font_Unit),
                    byte.Parse(s_Font_GdiCharSet),
                    bool.Parse(s_Font_GdiVertialFont));
                m_FontColor = Color.FromName(s_FontColor);
                m_Size.Width = int.Parse(s_Size_Width);
                m_Size.Height = int.Parse(s_Size_Height);
                m_Opacity = double.Parse(s_Opacity);
                m_Border = (BorderStyle)Enum.Parse(typeof(FormBorderStyle), s_Border);
            }

            // <Ver 1.18.0>カタログに登録されている外観の数を取得するクラスメンバ
            // 外観の数を返す
            public int GetIndexCount()
            {
                Config Conf = new Config();
                return Conf.GetRegIndexCount();
            }

            // <Ver 1.18.0>全プロパティの一致をスキャンしてインデックスを取得するクラスメンバ
            // 全てを検索する場合は arg = 0, 特定のインデックスの一致を調べるには arg = index
            // カタログ内で見つかればそのインデックス、見つからなければ 0 を返す
            public int UpdateIndex(int arg)
            {
                // 現在表示されている外観プロパティの文字列と配列を取得する
                UpdatePropertyStrings();
                SetaReg();

                int current_index = 0, index, start, end;
                if (arg == 0) { start = 1; end = MaxIndex; }
                else { start = end = arg; }
                
                for (index = start; index <= end; index++)
                {
                    bool consistent = true;
                    for (int j = 0; j < 12; j++)
                    {
                        consistent = (((string[])alReg[index - 1])[j] == a_reg[j].ToString()) && consistent;
                    }
                    if (consistent)
                    {
                        current_index = index;
                        break;
                    }
                }
                return current_index;
            }

            // <Ver 1.18.0>INIファイルのカタログ情報でArrayListを初期化するクラスメンバ
            //  現在表示外観のインデックスを返す
            public int InitializeIndex()
            {
                int ret = GetIndexCount();  // 最大インデックス数をINIファイルから取得
                if ( ret > 0)
                {
                    // 何か登録されている場合
                    MaxIndex = ret;
                    Config Conf = new Config();
                    for (int index = 1; index <= MaxIndex; index++)
                    {
                        if (Conf.GetRegInfo(index)) // indexに対応するプロパティ文字列をINIファイルから取得
                        {
                            SetaReg();  // プロパティ文字列を a_reg[] に入れる
                            alReg.Add(new string[12]);  // ArrayList alReg に空の a_reg[] を追加
                            for (int j=0; j<12; j++)    // ArrayList alReg に 現在表示のプロパティを入れる
                            {
                                ((string[])alReg[alReg.Count - 1])[j] = a_reg[j].ToString();
                            }
                        }
                    }
                }
                else if (ret == 0)
                {
                    // 何も登録されていない場合
                    CurrentIndex = 0;    // 現在のインデックスを 0 に初期化
                }
                else
                {
                    MessageBox.Show("外観情報の読み込みに失敗しました.");
                }

                // 現在表示外観のカタログインデックスを取得する
                CurrentIndex = UpdateIndex(0);

                return MaxIndex;
            }

            // <Ver 1.18.0>現在の外観プロパティがカタログになければ ArrayListに追加するクラスメンバ
            // カタログに既存の外観なら 0 を返し、無ければ新しいインデックスを返す
            public int AddIndex()
            {
                int index = UpdateIndex(0); // 追加しようとする現在表示外観のインデックスを取得
                if ( index == 0)
                {
                    // 登録しようとするプロパティがカタログに無い場合
                    CurrentIndex = ++MaxIndex;    // 最大/現在のインデックスをインクリメント
                    alReg.Add(new string[12]);
                    for (int i = 0; i < 12; i++)
                    {
                        ((string[])alReg[alReg.Count - 1])[i] = a_reg[i].ToString();
                    }
                }
                else
                {
                    // カタログに既存の場合は、見つかったインデックス番号で mreg_indexを更新する
                    CurrentIndex = index;
                }
                return CurrentIndex;
            }

            // <Ver 1.18.0>カタログにある外観を削除するクラスメンバ
            // 削除したインデックスを変返す
            public int DeleteIndex(int ret)
            {
                // ret ( =index) が 0 でない時のみ削除する
                if (ret > 0)
                {
                    alReg.RemoveAt(ret - 1);
                    //s_MaxIndex = GetIndexCount().ToString();

                    Config Conf = new Config();
                    for (int r_index=1, w_index=1; r_index <= MaxIndex; r_index++, w_index++)
                    {
                        if (r_index == ret)
                        {
                            if (++r_index > MaxIndex)
                                break;
                        }
                        Conf.GetRegInfo(r_index);
                        Conf.SetRegInfo(w_index);
                    }
                    DispChangebyIndex(ret - 1);
                    Conf.RemoveRegInfo(MaxIndex--);
                    Conf.SetRegIndexCount(MaxIndex);
                }
                return ret;
            }
            
            // <Ver 1.18.0>インデックスに応じた外観プロパティを設定するクラスメンバ
            public void DispChangebyIndex(int index)
            {
                GetRegItem(index);

                if (Display.m_normDisp)
                {
                    Display.m_NormBackColor = s_BackColor;
                    Display.m_NormFont = m_Font;
                    Display.m_NormFontColor = s_FontColor;
                    Display.m_DispNormSize = m_Size;
                    Display.m_NormOpacity = m_Opacity;
                    Display.m_NormBorder = s_Border;
                }
                else
                {
                    Display.m_SetAlarmBackColor = s_BackColor;
                    Display.m_SetAlarmFont = m_Font;
                    Display.m_SetAlarmFontColor = s_FontColor;
                    Display.m_DispSetAlarmSize = m_Size;
                    Display.m_SetAlarmOpacity = m_Opacity;
                    Display.m_SetAlarmBorder = s_Border;
                }
            }
        }
        // ========== Registerクラス ここまで ========== //
        
        // Registerクラスのインスタンスを生成する
        Register Reg = new Register();

        // <Ver 1.18.0>INIファイルに追加/削除する文字列
        public static string s_MaxIndex;
        public static string s_BackColor;
        public static string s_Font_Name;
        public static string s_Font_Size;
        public static string s_Font_Style;
        public static string s_Font_Unit;
        public static string s_Font_GdiCharSet;
        public static string s_Font_GdiVertialFont;
        public static string s_FontColor;
        public static string s_Size_Width;
        public static string s_Size_Height;
        public static string s_Opacity;
        public static string s_Border;

        // <Ver 1.18.0>外観変更フラグ
        public static bool m_ProertirsChanged;

        // <Ver 1.18.0>通常時の外観/アラーム設定時の外観の変更追跡フラグ
        public static bool reg_status = Display.m_normDisp;

        // <Ver 1.18.0>Msgダイアログ表示に使用する文字列変数
        public static string Msg;

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// 外観の登録ダイアログ描画のイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Disp_Register_Load(object sender, EventArgs e)
        {
            // このDispRegフォームを外観設定ダイアログや時計の周りに重ならないように表示する
            MoveDispRegForm();
            m_ProertirsChanged = false;
            this.trackBar_Reg.Select();
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// 外観の登録フォームを外観設定ダイアログや時計の周りに重ならないように表示する
        /// </summary>
        private void MoveDispRegForm()
        {
            // 登録フォームがあるディスプレイの高さと幅を取得
            int screenH = System.Windows.Forms.Screen.GetBounds(this).Height;
            int screenW = System.Windows.Forms.Screen.GetBounds(this).Width;
            // 外観設定フォーム (ctrl フォーム) の左上の位置、幅と高さを取得
            var ctrl_left = this.Owner.Location.X;
            var ctrl_top = this.Owner.Location.Y;
            var ctrl_width = this.Owner.Width;
            var ctrl_height = this.Owner.Height;
            // 時計の位置、幅と高さ
            int disp_left = Display.DispInstance.Left;
            int disp_top = Display.DispInstance.Top;
            int disp_width = Display.DispInstance.Width;
            int disp_height = Display.DispInstance.Height;
            // 設定位置 (x, y)からのマージン
            //int x_margin = 0;
            int y_margin = 10; 

            // 登録フォームのデフォルトの位置(外観設定フォームに対して時計のある方)
            int x=0, y=0;
            if (disp_left + disp_width <= ctrl_left)
            {
                // 時計が外観設定フォームの左にある時
                x = ctrl_left - this.Width;
                if (x  < 0)
                // 登録フォームがスクリーンの左からはみ出す時
                {
                    x = ctrl_left + ctrl_width;
                    y = ctrl_top;
                }
                else
                // 登録フォームがスクリーンの左からはみ出さない時
                {
                    y = ctrl_top + disp_height + y_margin;
                    if (disp_top > ctrl_top)
                    // 時計が下にある時
                    {
                        y = ctrl_top;
                    }
                }
            }
            else
            {
                // 時計が外観設定フォームの右にある時
                x = ctrl_left + ctrl_width;
                if (x + this.Width >= screenW)
                //登録フォームがスクリーンの右からはみ出す時
                {
                    x = ctrl_left - this.Width;
                    y = ctrl_top;
                }
                //登録フォームがスクリーンの右からはみ出さない時
                else
                {
                    y = ctrl_top + disp_height + y_margin;
                    if (disp_top > ctrl_top)
                    // 時計が下にある時
                    {
                        y = ctrl_top;
                    }
                }
            }
            // 外観登録フォームと時計の周りに移動させる
            this.Location = new Point(x, y);
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// コントロールのプロパティを更新する
        /// </summary>
        private void UpdateControls()
        {
            int max_index = Reg.MaxIndex;
            if (max_index > 0)
            {
                // トラックバーを有効にする
                this.trackBar_Reg.Enabled = true;
                if (Reg.CurrentIndex == 0)
                {
                    // [追加]ボタンを有効にする
                    this.button_regAdd.Enabled = true;
                    // [削除]ボタンを無効にする
                    this.button_regDel.Enabled = false;
                    this.label_Note.Text = "追加用のプロパティ";
                }
                else
                {
                    // [追加]ボタンを無効にする
                    this.button_regAdd.Enabled = false;
                    // [削除]ボタンを有効にする
                    this.button_regDel.Enabled = true;
                    this.label_Note.Text = "カタログ " + Reg.GetIndexString();
                }
            }
            else // max_index == 0 の時
            {
                // トラックバーを無効にする
                this.trackBar_Reg.Enabled = false;
                // [追加]ボタンを有効にする
                this.button_regAdd.Enabled = true;
                // [削除]ボタンを無効にする
                this.button_regDel.Enabled = false;
                this.label_Note.Text = "追加用のプロパティ";
            }
        }

        private void ShowInstruction()
        {
            string msg;
            if (Reg.CurrentIndex > 0)
            {
                if (Display.m_normDisp)
                {
                    msg = "[通常] ";
                    this.label_Help.ForeColor = Color.Black;
                }
                else
                {
                    msg = "[アラーム設定時] ";
                    this.label_Help.ForeColor = Color.Red;
                }
                msg += "の外観に即反映";
            }
            else
            {
                if (Display.m_normDisp)
                    this.label_Help.ForeColor = Color.Black;
                else
                    this.label_Help.ForeColor = Color.Red;
                msg = "そのまま/カスタマイズ後、カタログ追加可能.";
            }
            this.label_Help.Text = msg;
            
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// 外観カタログを入れ替える
        /// </summary>
        private void swapCatalog()
        {
            reg_status = Display.m_normDisp; // statusフラグをリセットする
            int index = Reg.UpdateIndex(0);

            // トラックバーの値を更新する
            this.trackBar_Reg.Value = index;
            // インデックスラベルを更新する
            this.label_Index.Text = Reg.SetIndexString(index);
            
            // コントロールを更新する
            UpdateControls();
            // メッセージ表示
            ShowInstruction();
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// 外観のプロパティをINIファイルに保存する
        /// </summary>
        private void SaveProperties()
        {
            Ctrl.m_PropertyChanged = false;
            int index = Reg.CurrentIndex;
            Config Conf = new Config();
            if ( index == 0)
            {
                Conf.SetIniFile();
            }
            else
            {
                // 外観情報を文字列に変換する(最大インデックス番号も取得)
                Reg.UpdatePropertyStrings();
                // 外観情報をINIファイルに保存する
                Conf.SetRegInfo(index);
            }
        }

        /// <summary>
        /// <Ver 1.17.6></Ver>
        /// 外観/プロパティ更新関連のフラグを追跡するタイマーイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer_Reg_Tick(object sender, EventArgs e)
        {
            // reg_sattus フラグが立てばトラックバーと外観カタログを反映させる
            if (reg_status != Display.m_normDisp)
                swapCatalog();

            // アラーム発動からの復帰フラグが立てば DispRegフォームを移動する
            if (Display.m_alarm_mode)
                MoveDispRegForm();

            // プロパティ変更フラグが立てばINIファイルに保存してカタログフォームを移動する
            if (Ctrl.m_PropertyChanged)
            {
                // プロパティをINIファイルに保存する
                SaveProperties();
                // 時計のサイズが変わる可能性があるので、カタログ画面を再配置する
                MoveDispRegForm();
            }
        }

        /// <summary>
        /// <ver 1.18.0></ver>
        ///  外観カタログ画面を状況に応じて更新する
        /// </summary>
        private void Initialize()
        {
            // INIファイルから外観情報を読み出し、最大登録数を取得する
            int max_index = Reg.InitializeIndex();

            // トラックバーのMaximumを更新する
            this.trackBar_Reg.Maximum = max_index;
            // トラックバーの値を更新する
            // <Ver 1.18.1> 現在のインデックスを 0 で初期化
            this.trackBar_Reg.Value = Reg.CurrentIndex = 0;
            // インデックスラベルを更新する
            this.label_Index.Text = Reg.GetIndexString();

            // コントロールを更新する
            UpdateControls();
            // プロパティ変更のフラグを変える
            Display.m_ProertirsChanged = true;
            // メッセージ表示
            ShowInstruction();
            // トラックバーをフォーカス
            this.trackBar_Reg.Select();
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// [追加]ボタンクリックのイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_regAdd_Click(object sender, EventArgs e)
        {
            // 現在の外観がカタログにないプロパティセットを持つ場合は alReg の最後に追加し、
            // そのインデックスを返す。
            // 既存の外観の場合はカタログインデックスを返す
            int index = Reg.AddIndex();

            // トラックバーを有効にする
            this.trackBar_Reg.Enabled = true;
            // [追加]ボタンを無効にする
            this.button_regAdd.Enabled = false;
            // [削除]ボタンを有効にする
            this.button_regDel.Enabled = true;
            this.label_Note.Text = "カタログ " + Reg.GetIndexString();

            Config Conf = new Config();
            // index が INIファイルの最大インデックス以下の場合は、既存の外観があると判断して 
            // トラックバーを更新するのみ
            if (index <= Conf.GetRegIndexCount())
            {
                // トラックバーの値を更新する
                this.trackBar_Reg.Value = index;
                // インデックスラベルを更新する
                this.label_Index.Text = Reg.GetIndexString();

                // メッセージ表示
                Msg = "既にカタログに有るので、追加しません.";
                DispRegMsg dlg = new DispRegMsg();
                dlg.ShowDialog(this);
            }
            else
            // カタログにない新しい外観の場合は、トラックバーを更新して、INIファイルに保存する
            {
                // トラックバーのMaximumを更新/設定する
                this.trackBar_Reg.Maximum = Reg.MaxIndex;
                // トラックバーの値を更新する
                this.trackBar_Reg.Value = index;
                // インデックスラベルを更新する
                this.label_Index.Text = Reg.GetIndexString();

                // 新しい外観プロパティをINIファイルに書き込む
                // 外観情報を文字列に変換する(最大インデックス数も取得する)
                Reg.UpdatePropertyStrings();
                // 外観情報をINIファイルに保存する(最大インデックス数も保存)
                Conf.SetRegInfo(Reg.MaxIndex);
            }

            // プロパティ変更フラグを立てる
            m_ProertirsChanged = true;
            this.trackBar_Reg.Select();
            // メッセージ表示
            ShowInstruction();
            // トラックバーをフォーカス
            this.trackBar_Reg.Select();
        }

        /// <summary>
        /// <ver 1.18.0></ver>
        /// [削除]ボタンクリックのイベントハンドラ 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_regDel_Click(object sender, EventArgs e)
        {
            // ArrayListとINIファイルから削除し、削除したインデックスを返す
            int index = Reg.DeleteIndex(Reg.CurrentIndex) - 1;

            // トラックバーのMaximumを更新/設定する
            this.trackBar_Reg.Maximum = Reg.MaxIndex;
            // トラックバーの値を更新する
            this.trackBar_Reg.Value = index;
            // インデックスラベルを更新する
            this.label_Index.Text = Reg.SetIndexString(index);

            // コントロールを更新化する
            UpdateControls();
            // プロパティ変更フラグを立てる
            m_ProertirsChanged = true;
            this.trackBar_Reg.Select();
            // メッセージ表示
            ShowInstruction();
            // トラックバーをフォーカス
            this.trackBar_Reg.Select();
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// トラックバーを動かした時のイベントハンドラ 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trackBar_Reg_Scroll(object sender, EventArgs e)
        {
            int ret = Reg.UpdateIndex(0);
            int index = trackBar_Reg.Value;
            
            // インデックスラベルを更新する
            this.label_Index.Text = Reg.SetIndexString(index);
            // ArryListから指定された index に相当する外観に変更する
            Reg.DispChangebyIndex(index);

            // コントロールを更新する
            UpdateControls();
            // プロパティ変更フラグを立てる
            m_ProertirsChanged = true;
            // メッセージ表示
            ShowInstruction();
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// [閉じる]ボタンクリックのイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_OK_Click(object sender, EventArgs e)
        {
            Dispose();
        }
    }
}
